@@ -13,35 +13,95 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 # OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: ipv6.py,v 1.5 2003/06/30 06:12:47 halley Exp $
+# $Id: ipv6.py,v 1.9 2003/07/20 22:14:41 halley Exp $
 
 """IPv6 helper functions."""
 
 import re
 
 import dns.exception
+import dns.ipv4
 
-v4_ending = re.compile(r'(.*):(\d+)\.(\d+)\.(\d+)\.(\d+)$')
-colon_colon_start = re.compile(r'::.*')
-colon_colon_end = re.compile(r'.*::$')
+_leading_zero = re.compile(r'0+([0-9a-f]+)')
 
 def inet_ntoa(address):
-    """Convert a network format IPv6 address into text."""
+    """Convert a network format IPv6 address into text.
+
+    @param address: the binary address
+    @type address: string
+    @rtype: string
+    @raises ValueError: the address isn't 16 bytes long
+    """
 
     if len(address) != 16:
-        raise dns.exception.SyntaxError
-    hex = address.encode('hex_codec').upper()
+        raise ValueError, "IPv6 addresses are 16 bytes long"
+    hex = address.encode('hex_codec')
     chunks = []
     i = 0
     l = len(hex)
     while i < l:
-        chunks.append(hex[i : i + 4])
+        chunk = hex[i : i + 4]
+        # strip leading zeros.  we do this with an re instead of
+        # with lstrip() because lstrip() didn't support chars until
+        # python 2.2.2
+        m = _leading_zero.match(chunk)
+        if not m is None:
+            chunk = m.group(1)
+        chunks.append(chunk)
         i += 4
-    hex = ':'.join(chunks)
+    #
+    # Compress the longest subsequence of 0-value chunks to ::
+    #
+    best_start = 0
+    best_len = 0
+    last_was_zero = False
+    for i in xrange(8):
+        if chunks[i] != '0':
+            if last_was_zero:
+                end = i
+                current_len = end - start
+                if current_len > best_len:
+                    best_start = start
+                    best_len = current_len
+                last_was_zero = False
+        elif not last_was_zero:
+            start = i
+            last_was_zero = True
+    if last_was_zero:
+        end = 8
+        current_len = end - start
+        if current_len > best_len:
+            best_start = start
+            best_len = current_len
+    if best_len > 0:
+        if best_start == 0 and \
+           (best_len == 6 or
+            best_len == 5 and chunks[5] == 'ffff'):
+            # We have an embedded IPv4 address
+            if best_len == 6:
+                prefix = '::'
+            else:
+                prefix = '::ffff:'
+            hex = prefix + dns.ipv4.inet_ntoa(address[12:])
+        else:
+            hex = ':'.join(chunks[:best_start]) + '::' + \
+                  ':'.join(chunks[best_start + best_len:])
+    else:
+        hex = ':'.join(chunks)
     return hex
 
+_v4_ending = re.compile(r'(.*):(\d+)\.(\d+)\.(\d+)\.(\d+)$')
+_colon_colon_start = re.compile(r'::.*')
+_colon_colon_end = re.compile(r'.*::$')
+
 def inet_aton(text):
-    """Convert a text format IPv6 address into network format."""
+    """Convert a text format IPv6 address into network format.
+    
+    @param text: the textual address
+    @type text: string
+    @rtype: string
+    @raises dns.exception.SyntaxError: the text was not properly formatted
+    """
     
     #
     # Our aim here is not something fast; we just want something that works.
@@ -52,7 +112,7 @@ def inet_aton(text):
     #
     # Get rid of the icky dot-quad syntax if we have it.
     #
-    m = v4_ending.match(text)
+    m = _v4_ending.match(text)
     if not m is None:
         text = "%s:%04x:%04x" % (m.group(1),
                                  int(m.group(2)) * 256 + int(m.group(3)),
@@ -61,17 +121,16 @@ def inet_aton(text):
     # Try to turn '::<whatever>' into ':<whatever>'; if no match try to
     # turn '<whatever>::' into '<whatever>:'
     #
-    m = colon_colon_start.match(text)
+    m = _colon_colon_start.match(text)
     if not m is None:
         text = text[1:]
     else:
-        m = colon_colon_end.match(text)
+        m = _colon_colon_end.match(text)
         if not m is None:
             text = text[:-1]
     #
     # Now canonicalize into 8 chunks of 4 hex digits each
     #
-
     chunks = text.split(':')
     l = len(chunks)
     if l > 8:
