@@ -13,7 +13,7 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 # OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: resolver.py,v 1.13 2003/06/30 05:13:25 halley Exp $
+# $Id: resolver.py,v 1.15 2003/07/12 02:29:39 halley Exp $
 
 """DNS stub resolver.
 
@@ -59,7 +59,7 @@ class Answer(object):
     resolution.
 
     For convenience, the answer is iterable.  "for a in answer" is
-    equivalent to "for a in answer.rdataset".
+    equivalent to "for a in answer.rrset".
 
     Note that CNAMEs or DNAMEs in the response may mean that answer
     node's name might not be the query name.
@@ -72,10 +72,8 @@ class Answer(object):
     @type rdclass: int
     @ivar response: The response message
     @type response: dns.message.Message object
-    @ivar node: The node of the answer
-    @type node: dns.node.Node object
-    @ivar rdataset: The answer
-    @type rdataset: dns.rdataset.Rdataset object
+    @ivar rrset: The answer
+    @type rrset: dns.rrset.RRset object
     @ivar expiration: The time when the answer expires
     @type expiration: float (seconds since the epoch)
     """
@@ -85,52 +83,54 @@ class Answer(object):
         self.rdclass = rdclass
         self.response = response
         min_ttl = -1
+        rrset = None
         for count in xrange(0, 15):
-            node = None
-            rds = None
             try:
-                node = response.find_node(response.answer, qname)
-                try:
-                    rds = node.find_rdataset(rdclass, rdtype)
-                    if min_ttl == -1 or rds.ttl < min_ttl:
-                        min_ttl = rds.ttl
-                    break
-                except KeyError:
-                    if rdtype != dns.rdatatype.CNAME:
-                        try:
-                            crds = node.find_rdataset(rdclass,
-                                                      dns.rdatatype.CNAME)
-                            if min_ttl == -1 or crds.ttl < min_ttl:
-                                min_ttl = crds.ttl
-                            for rd in crds:
-                                qname = rd.target
-                                break
-                            continue
-                        except KeyError:
-                            raise NoAnswer
+                rrset = response.find_rrset(response.answer, qname,
+                                            rdclass, rdtype)
+                if min_ttl == -1 or rrset.ttl < min_ttl:
+                    min_ttl = rrset.ttl
+                break
             except KeyError:
+                if rdtype != dns.rdatatype.CNAME:
+                    try:
+                        crrset = response.find_rrset(response.answer,
+                                                     qname,
+                                                     rdclass,
+                                                     dns.rdatatype.CNAME)
+                        if min_ttl == -1 or crrset.ttl < min_ttl:
+                            min_ttl = crrset.ttl
+                        for rd in crrset:
+                            qname = rd.target
+                            break
+                        continue
+                    except KeyError:
+                        raise NoAnswer
                 raise NoAnswer
-        if rds is None:
+        if rrset is None:
             raise NoAnswer
-        self.node = node
-        self.rdataset = rds
+        self.rrset = rrset
         self.expiration = time.time() + min_ttl
 
     def __getattr__(self, attr):
         if attr == 'name':
-            return self.node.name
+            return self.rrset.name
         elif attr == 'ttl':
-            return self.rdataset.ttl
+            return self.rrset.ttl
         elif attr == 'covers':
-            return self.rdataset.covers
+            return self.rrset.covers
+        elif attr == 'rdclass':
+            return self.rrset.rdclass
+        elif attr == 'rdtype':
+            return self.rrset.rdtype
         else:
             raise AttributeError, attr
 
     def __len__(self):
-        return len(self.rdataset)
+        return len(self.rrset)
 
     def __iter__(self):
-        return iter(self.rdataset)
+        return iter(self.rrset)
 
 class Cache(object):
     """Simple DNS answer cache.
@@ -295,7 +295,11 @@ class Resolver(object):
             want_close = False
         try:
             for l in f:
+                if len(l) == 0 or l[0] == '#' or l[0] == ';':
+                    continue
                 tokens = l.split()
+                if len(tokens) == 0:
+                    continue
                 if tokens[0] == 'nameserver':
                     self.nameservers.append(tokens[1])
                 elif tokens[0] == 'domain':
