@@ -13,7 +13,7 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 # OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: tokenizer.py,v 1.19 2003/06/24 02:37:00 halley Exp $
+# $Id: tokenizer.py,v 1.20 2003/07/16 10:42:43 halley Exp $
 
 """Tokenize DNS master file format"""
 
@@ -42,6 +42,11 @@ QUOTED_STRING = 4
 COMMENT = 5
 DELIMITER = 6
 
+class UngetBufferFull(dns.exception.DNSException):
+    """Raised when an attempt is made to unget a token when the unget
+    buffer is full."""
+    pass
+    
 class Tokenizer(object):
     """A DNS master file format tokenizer.
 
@@ -65,7 +70,8 @@ class Tokenizer(object):
     @ivar eof: This variable is true if the tokenizer has encountered EOF.
     @type eof: bool
     @ivar delimiters: The current delimiter dictionary.
-    @type delimiters: dict"""
+    @type delimiters: dict
+    """
     
     def __init__(self, f=sys.stdin):
         """Initialize a tokenizer instance.
@@ -73,7 +79,8 @@ class Tokenizer(object):
         @param f: The file to tokenize.  The default is sys.stdin.
         This parameter may also be a string, in which case the tokenizer
         will take its input from the contents of the string.
-        @type f: file or string"""
+        @type f: file or string
+        """
         
         if isinstance(f, str):
             f = cStringIO.StringIO(f)
@@ -85,9 +92,10 @@ class Tokenizer(object):
         self.eof = False
         self.delimiters = _DELIMITERS
 
-    def get_char(self):
+    def _get_char(self):
         """Read a character from input.
-        @rtype: string"""
+        @rtype: string
+        """
         
         if self.ungotten_char is None:
             if self.eof:
@@ -101,7 +109,7 @@ class Tokenizer(object):
             self.ungotten_char = None
         return c
 
-    def unget_char(self, c):
+    def _unget_char(self, c):
         """Unget a character.
 
         The unget buffer for characters is only one character large; it is
@@ -109,9 +117,12 @@ class Tokenizer(object):
         empty.
         
         @param c: the character to unget
-        @type c: string"""
+        @type c: string
+        @raises UngetBufferFull: there is already an ungotten char
+        """
         
-        assert self.ungotten_char is None
+        if not self.ungotten_char is None:
+            raise UngetBufferFull
         self.ungotten_char = c
 
     def skip_whitespace(self):
@@ -122,14 +133,15 @@ class Tokenizer(object):
 
         If the tokenizer is in multiline mode, then newlines are whitespace.
 
-        @rtype: int"""
+        @rtype: int
+        """
         
         skipped = 0
         while True:
-            c = self.get_char()
+            c = self._get_char()
             if c != ' ' and c != '\t':
                 if (c != '\n') or not self.multiline:
-                    self.unget_char(c)
+                    self._unget_char(c)
                     return skipped
             skipped += 1
 
@@ -142,7 +154,10 @@ class Tokenizer(object):
         @param want_comment: If True, return a COMMENT token if the
         first token read is a comment.  The default is False.
         @type want_comment: bool
-        @rtype: (int, string) tuple"""
+        @rtype: (int, string) tuple
+        @raises dns.exception.UnexpectedEnd: input ended prematurely
+        @raises dns.exception.SyntaxError: input was badly formed
+        """
         
         if not self.ungotten_token is None:
             token = self.ungotten_token
@@ -161,7 +176,7 @@ class Tokenizer(object):
         token = ''
         ttype = IDENTIFIER
         while True:
-            c = self.get_char()
+            c = self._get_char()
             if c == '' or c in self.delimiters:
                 if c == '' and self.quoting:
                     raise dns.exception.UnexpectedEnd
@@ -191,12 +206,12 @@ class Tokenizer(object):
                         return (EOL, '\n')
                     elif c == ';':
                         while 1:
-                            c = self.get_char()
+                            c = self._get_char()
                             if c == '\n' or c == '':
                                 break
                             token += c
                         if want_comment:
-                            self.unget_char(c)
+                            self._unget_char(c)
                             return (COMMENT, token)
                         elif c == '':
                             if self.multiline:
@@ -216,18 +231,18 @@ class Tokenizer(object):
                         token = c
                         ttype = DELIMITER
                 else:
-                    self.unget_char(c)
+                    self._unget_char(c)
                 break
             elif self.quoting:
                 if c == '\\':
-                    c = self.get_char()
+                    c = self._get_char()
                     if c == '':
                         raise dns.exception.UnexpectedEnd
                     if c.isdigit():
-                        c2 = self.get_char()
+                        c2 = self._get_char()
                         if c2 == '':
                             raise dns.exception.UnexpectedEnd
-                        c3 = self.get_char()
+                        c3 = self._get_char()
                         if c == '':
                             raise dns.exception.UnexpectedEnd
                         if not (c2.isdigit() and c3.isdigit()):
@@ -250,14 +265,18 @@ class Tokenizer(object):
         empty.
         
         @param token: the token to unget
-        @type token: string"""
+        @type token: string
+        @raises UngetBufferFull: there is already an ungotten char
+        """
 
-        assert self.ungotten_token is None
+        if not self.ungotten_token is None:
+            raise UngetBufferFull
         self.ungotten_token = token
 
     def next(self):
         """Return the next item in an iteration.
-        @rtype: (int, string)"""
+        @rtype: (int, string)
+        """
         
         token = self.get()
         if token[0] == EOF:
@@ -273,7 +292,8 @@ class Tokenizer(object):
         """Read the next token and interpret it as an integer.
         
         @raises dns.exception.SyntaxError:
-        @rtype: int"""
+        @rtype: int
+        """
         
         (ttype, value) = self.get()
         if ttype != IDENTIFIER:
@@ -287,7 +307,8 @@ class Tokenizer(object):
         integer.
         
         @raises dns.exception.SyntaxError:
-        @rtype: int"""
+        @rtype: int
+        """
         
         value = self.get_int()
         if value < 0 or value > 255:
@@ -300,7 +321,8 @@ class Tokenizer(object):
         integer.
         
         @raises dns.exception.SyntaxError:
-        @rtype: int"""
+        @rtype: int
+        """
         
         value = self.get_int()
         if value < 0 or value > 65535:
@@ -313,7 +335,8 @@ class Tokenizer(object):
         integer.
                 
         @raises dns.exception.SyntaxError:
-        @rtype: int"""
+        @rtype: int
+        """
         
         (ttype, value) = self.get()
         if ttype != IDENTIFIER:
@@ -330,7 +353,8 @@ class Tokenizer(object):
         """Read the next token and interpret it as a string.
                 
         @raises dns.exception.SyntaxError:
-        @rtype: string"""
+        @rtype: string
+        """
         
         (ttype, t) = self.get()
         if ttype != IDENTIFIER and ttype != QUOTED_STRING:
@@ -353,7 +377,8 @@ class Tokenizer(object):
         EOF.
 
         @raises dns.exception.SyntaxError:
-        @rtype: string"""
+        @rtype: string
+        """
         
         (ttype, t) = self.get()
         if ttype != EOL and ttype != EOF:
